// ----------------------------------------------------------------------------
// Origin:    European Cooperation for Space Standardization (ECSS)
// Project:   E-ST-40-07 Simulation Modelling Platform (SMP)
// File:      Smp/ISimulatorL2.h
// Version:   Issue 1.0 from 30/09/2024
// SPDX-License-Identifier: BSD-2-Clause
// SPDX-FileCopyrightText: Copyright 2024 Telespazio Germany GmbH
// ----------------------------------------------------------------------------

#ifndef SMP_ISIMULATORL2_H_
#define SMP_ISIMULATORL2_H_

// ----------------------------------------------------------------------------
// ---------------------------- Include Header Files --------------------------
// ----------------------------------------------------------------------------

#include "Smp/FileNotFound.h"
#include "Smp/ISimulator.h"
#include "Smp/PrimitiveTypes.h"

// ----------------------------------------------------------------------------
// ------------------------ Types and Interfaces ------------------------------
// ----------------------------------------------------------------------------

/// SMP standard types and interfaces.
namespace Smp
{
    /// SMP Level 2 extension of the ISimulator interface. It provides
    /// convenience methods to load SMPL2 compliant Assemblies, Schedules and
    /// Linkbases.
    class ISimulatorL2 :
        public virtual Smp::ISimulator
    {
    public:
        /// Virtual destructor to release memory.
        virtual ~ISimulatorL2() noexcept = default;

        /// This operation loads an SMPL2 Assembly file and tries to assemble
        /// all contained models and connections. The given assembly is
        /// validated during this construction step and will fail if components
        /// are missing or the file is malformated.
        /// @param   assemblyPath Path to the assembly file to load.
        /// This needs to be a valid path name given the constraints of the
        /// operating system.
        /// @throws  Smp::FileNotFound
        virtual void LoadAssembly(
            Smp::String8 assemblyPath) = 0;

        /// This operation loads an SMPL2 LinkBase file and tries to connect
        /// all referenced models. The given Linkbase is validated during this
        /// construction step and will fail if components are missing or the
        /// file is malformated.
        /// @param   linkBasePath Path to the LinkBase file to load.
        /// This needs to be a valid path name given the constraints of the
        /// operating system.
        /// @throws  Smp::FileNotFound
        virtual void LoadLinkBase(
            Smp::String8 linkBasePath) = 0;

        /// This operation loads a SMPL2 Schedule file and tries to schedule
        /// all events. The given schedule is validated during this processing
        /// step and will fail if events are missing or the file is malformated.
        /// @param   schedulePath Path to the schedule file to load.
        /// This needs to be a valid path name given the constraints of the
        /// operating system.
        /// @throws  Smp::FileNotFound
        virtual void LoadSchedule(
            Smp::String8 schedulePath) = 0;
    };
}

#endif // SMP_ISIMULATORL2_H_
